@echo off 


rem last update was on June 27, 2014

rem Batch file to perform several tasks used to prepare Fallout NV vanilla textures for optimization using DDSopt
rem as per steps in the DDSopt guide (http://wiki.step-project.com/Guide:DDSopt#tab=DDSopt_Optimization).
rem It is only useful when being used with other steps described in this guide.
rem If this batch file is edited, this MUST BE DONE with an editor that does not add additional characters (e.g., line feed, carriage return)
rem these characters are often invisible but can sometimes cause the batch file to malfunction
rem DO NOT USE a default Windows text editor to edit this file
rem Editors like Notepad++ (which was used to create this file), TextPad, UltraEdit are fine
 
rem It expects that there is a Working directory containing the "Vanilla Extracted" directory
rem This directory should not be in any User Access Controlled (UAC) directory such as Program Files or Program Files (x86).
rem It expects that the BSA contents have already been extracted into folders in the  "Vanilla Extracted" directory
rem It  will work with folders for one or more of the extracted Fallout NV BSAs
rem The folder names for each extracted BSA are arbitrary, but one possible choice would be the following
rem For the basic Fallout NV use 3 folders, possible names are: Fallout_textures, Fallout_textures2, and Update_textures
rem If there are any optional DLC being used their BSAs should have also been extracted into "Vanilla Extracted"
rem potential folder names are "HonestHearts_textures", "DeadMoney_textures", "OldWorldBlues_textures", "LonesomeRoad_textures", and "GunRUnnersArsenal_textures" (based on the DLC being used).
rem If the PreOrder Packs are used possible folder names are:
rem "Caravan_textures", "Classic_textures", "Mercenary_textures", and "Tribal_textures" (or a subset of these if all the PreOrder Packs are not used)
rem
rem
rem While the batch file runs there will be a command window that echoes the commands
rem This allows monitoring the progress of the batch processing if desired, although it is not required.
 
rem the initial 2 lines should ensure that the system provides the correct working directory name in %cd%
rem This is needed for a small percentage of systems, probably because of interaction with Windows UAC on those systems

REM tests added 8/18/2013 to make sure the batch files are in the right folder and the folder structures are correct
rem 2/4/2014 test added for presence of windows\system32\robocopy.exe



IF NOT EXIST "%Windir%\system32\robocopy.exe" echo ERROR: The Robocopy system program isn't available; fix this before proceeding

@setlocal enableextensions
@cd /d "%~dp0"
 
echo.
echo.
echo The system says the Working Directory is   %cd%
echo This directory should include the "Vanilla Extracted" directory
echo If any of the above is incorrect terminate the processing and fix the problem
echo.

rem test added 12/19/2013 and modified 1/13/2014 to test for administrative privileges
rem taken from http://stackoverflow.com/questions/4051883/batch-script-how-to-check-for-admin-rights
rem administrative privilege isn't usually needed
:testPrivilegies
    set guid=%random%%random%-%random%-%random%-%random%-%random%%random%%random%
    mkdir %WINDIR%\%guid%>nul 2>&1
    rmdir %WINDIR%\%guid%>nul 2>&1
    IF NOT %ERRORLEVEL%==0 echo  Administrative privileges are not enabled; this is usually not a problem
        


IF NOT EXIST "%cd%\Vanilla Extracted" echo  Error: The Vanilla Extracted directory is not in the working directory; fix this before proceeding
echo.
echo.
echo  ---------------------
echo On some systems the batch file may take a long time (30 minutes or so) and might not seem to be working
echo If it starts successfully and gets through a step or two it is almost certainly working so be patient with it
echo it's doing the best it can
echo ---------------------
echo Hit enter to continue after the pause
pause

 
IF NOT EXIST "%cd%\Vanilla Ordinary Textures\NUL"  MD "Vanilla Ordinary Textures"
IF NOT EXIST "%cd%\Vanilla Exterior Textures\NUL"  MD "Vanilla Exterior Textures"
IF NOT EXIST "%cd%\Vanilla Exterior Normal Maps\NUL"  MD "Vanilla Exterior Normal Maps"
IF NOT EXIST "%cd%\Vanilla Optimized\NUL"  MD "Vanilla Optimized"
IF NOT EXIST "%cd%\Vanilla Body Textures\NUL"  MD "Vanilla Body Textures"

REM Get the starting time
set STARTTIME=%TIME%


@echo on



rem copy all non-interface textures to Vanilla Exterior Normal Maps, excluding 3 files that have problems when optimized
%Windir%\system32\robocopy "%cd%\Vanilla Extracted" "%cd%\Vanilla Exterior Normal Maps" *.dds /s /xf blackdoor.dds blackdoor_n.dds white.dds /xd *interface *terminals *pipboy3000 *pimpboy3billion  /nfl /ndl /njh /njs /ns /nc /np >>log.txt

rem move all non-exterior textures to Vanilla Body Textures
%Windir%\system32\robocopy "%cd%\Vanilla Exterior Normal Maps" "%cd%\Vanilla Body Textures" *.dds /s  /move /xd *architecture *landscape /nfl /ndl /njh /njs /ns /nc /np >>log.txt

rem move all exterior textures to Vanilla Exterior Textures except the Normal Map Textures
%Windir%\system32\robocopy "%cd%\Vanilla Exterior Normal Maps" "%cd%\Vanilla Exterior Textures" *.dds /s  /move /xf *_n.dds /nfl /ndl /njh /njs /ns /nc /np >>log.txt

rem move all non-body-related textures to Vanilla Ordinary Textures
%Windir%\system32\robocopy "%cd%\Vanilla Body Textures" "%cd%\Vanilla Ordinary Textures" *.dds /s  /move /xf *_msn.dds /xd *characters /nfl /ndl /njh /njs /ns /nc /np >>log.txt



@echo off


rem get time of completion
set ENDTIME=%TIME%

rem from http://stackoverflow.com/questions/9922498/calculate-time-difference-in-batch-file

rem Change formatting for the start and end times
for /F "tokens=1-4 delims=:.," %%a in ("%STARTTIME%") do (
    set /A "start=(((%%a*60)+1%%b %% 100)*60+1%%c %% 100)*100+1%%d %% 100"
)

for /F "tokens=1-4 delims=:.," %%a in ("%ENDTIME%") do (
    set /A "end=(((%%a*60)+1%%b %% 100)*60+1%%c %% 100)*100+1%%d %% 100"
)

rem Calculate the elapsed time by subtracting values
set /A elapsed=end-start

rem Format the results for output
set /A hh=elapsed/(60*60*100), rest=elapsed%%(60*60*100), mm=rest/(60*100), rest%%=60*100, ss=rest/100, cc=rest%%100
if %hh% lss 10 set hh=0%hh%
if %mm% lss 10 set mm=0%mm%
if %ss% lss 10 set ss=0%ss%
if %cc% lss 10 set cc=0%cc%

set DURATION=%hh%:%mm%:%ss%,%cc%


echo.
echo.
echo Start    : %STARTTIME%
echo Finish   : %ENDTIME%
echo          ---------------
echo Duration : %DURATION%

echo.
echo The batch file processing has finished.
echo  Check the log file (log.txt). The log file should not show any errors and should have 5 blank lines.
echo.
pause

