@echo off 


rem last update was on June 28, 2014
rem This is the first version for Fallout NV

rem 2/11/2014 fixed problem with some body-related tangent space normal maps remaining in "Mod Normal Maps"
 
rem Batch file to perform several tasks to prepare Fallout NV mod textures for optimization 
rem using DDSopt as per steps in the DDSopt guide (http://wiki.step-project.com/Guide:DDSopt#tab=Optimizing_the_STEP_Mods).
rem This batch processing supports being used incrementally; it works with a single mod or a set of mods.

rem It is only useful when being used with other steps described in this guide.
rem However, the batch file itself is not limited to mods in a STEP list; it works with any Skyrim mod.
rem If this batch file is edited, this MUST BE DONE with an editor that does not add additional characters (e.g., line feed, carriage return)
rem these characters are often invisible but can cause the batch file to malfunction
rem DO NOT USE a default Windows text editor to edit this file
rem Editors like Notepad++ (which was used to create this file), TextPad, UltraEdit are fine
 
rem It expects that there is a Working directory containing the "Mod Extracted" directory
rem This directory should not be in any User Access Controlled (UAC) directory such as Program Files or Program Files (x86).
rem It expects that the mod contents have already been extracted into folders in the  "Mod Extracted" directory
rem and that textures have been extracted from any BSA(s) present in the mod.
rem If a BSA contains only textures or textures are a major part of the BSA content it is best to fully extract the BSA and then remove the BSA after the optimization.
rem Note that the batch file will copy any BSAs present to the output. This is done since some mods have BSAs which don't contain
rem textures or which contain very little texture data. It is more efficient to let Fallout NV use such BSAs directly.
rem
rem The batch files ask whether exterior textures should be treated differently than other textures
rem If  "Y" is entered the exterior textures are copied to different folders so they can be optimized with different resolutions or formats.
rem If the mods do not contain a combination of exterior  and non-exterior textures then it's best to not  enter "Y "at the prompt.
rem

rem If mod files with two resolutions are present, exterior textures are taken from the lower resolution version of a mod,
rem and the other textures from the higher resolution version.
rem There must be 2 versions of the same mod with different texture resolutions, one in the '"Higher Res Mod Files'"directory
rem and one in the "Lower Res Mod Files" directory.
rem Any non-texture files in the mod are taken from the higher resolution version.
rem Ideally both mod folder names (the lower and higher res versions of the mod) should have the same name
rem so the textures are merged when optimized
rem The batch file doesn't use the mod file names so the file names for a mod don't need to be the same as the original mod file name.



rem Operation of the batch file

rem While the batch file runs there will be a command window that echoes the commands
rem This allows monitoring the progress of the batch processing if desired, although it is not required.
rem The output folder in "Mod Optimized" can then be archived and added it to the game.

rem This can be done with any or all of mods in the folder..

rem When the manual optimization steps using DDSopt are complete and the optimized textures are stored in archive files
rem and these files have been checked, the temporary directories
rem "Mod Ordinary Textures", "Mod Normal Maps", and , "Mod Body Textures", "Mod Exterior Textures", "Mod Exterior Normal Maps",
rem and their contents, can be deleted.



rem Ignore any ERROR Level 2 errors (The system cannot find the file specified) in the error log; these are not actual errors.
rem Errors with ERROR level >3 are significant.




rem test for existence of robocopy; added 2/4/2014
IF NOT EXIST "%Windir%\system32\robocopy.exe" echo ERROR: The Robocopy system program isn't available; fix this before proceeding
echo.
 
rem these 2 lines should ensure that the system provides the correct working directory name in %cd%
rem This is needed for a small percentage of systems, probably because of interaction with Windows UAC on those systems
@setlocal enableextensions
@cd /d "%~dp0"

@echo off
echo.
echo The system says the Working Directory is   %cd%
echo This directory should include the "Mod Extracted" or "Higher Res Mod Files" directories, or both
echo If the "Higher Res Mod Files" directory is used there must also be a  "Lower Res Mod Files" directory
echo folder names of the lower and higher res versions of mods in these two directories must be the same
echo If any of the above is incorrect terminate the processing and fix the problem

rem test added 12/19/2013 and modified 1/13/2014 to test for administrative privileges
rem taken from http://stackoverflow.com/questions/4051883/batch-script-how-to-check-for-admin-rights
rem administrative privilege isn't usually needed
:testPrivilegies
    set guid=%random%%random%-%random%-%random%-%random%-%random%%random%%random%
    mkdir %WINDIR%\%guid%>nul 2>&1
    rmdir %WINDIR%\%guid%>nul 2>&1
    IF NOT %ERRORLEVEL%==0 echo  Administrative privileges are not enabled; this is usually not a problem


IF NOT EXIST "%cd%\Mod Extracted" IF NOT EXIST "%cd%\Higher Res Mod Files" echo Error: There are no directories for extracted mods in the working directory; fix this before proceeding
IF EXIST "%cd%\Higher Res Mod Files" IF NOT EXIST "%cd%\Lower Res Mod Files" echo Error: Both "Higher Res Mod Files" and "Lower Res Mod Files" must be present for selecting different resolution files from a mod
echo.
echo.
echo additional tests complete
pause

rem create 3 folders that will be used then deleted
IF NOT EXIST "%cd%\Mod DDS\NUL" MD "Mod DDS"
IF NOT EXIST "%cd%\Mod TMP\NUL"  MD "Mod TMP"
IF NOT EXIST "%cd%\Mod TSN\NUL"  MD "Mod TSN"

rem create 4 output folders
IF NOT EXIST "%cd%\Mod Normal Maps\NUL"  MD "Mod Normal Maps"
IF NOT EXIST "%cd%\Mod Optimized\NUL"  MD "Mod Optimized"
IF NOT EXIST "%cd%\Mod Body Textures\NUL"  MD "Mod Body Textures"
IF NOT EXIST "%cd%\Mod Ordinary Textures\NUL"  MD "Mod Ordinary Textures"

REM Get the starting time
set STARTTIME=%TIME%


IF EXIST "%cd%\Mod Extracted" %Windir%\system32\robocopy "%cd%\Mod Extracted" "%cd%\Mod Optimized" *.* /s  /xf *.dds /nfl /ndl /njh /njs /ns /nc /np >>log.txt
rem copy all non-texture files to the output folder

:Ask
echo To separate external textures from other textures enter "Y" at the prompt
set /P INPUT=Type input: %=%
If /I "%INPUT%"=="y" goto exterior


@echo on

rem put model space normal maps in Mod Body Textures so uncompressed textures won't be corrupted or compressed
%Windir%\system32\robocopy "%cd%\Mod Extracted" "%cd%\Mod Body Textures" *_msn.dds /s /nfl /ndl /njh /njs /ns /nc /np >>log.txt
rem copy tangent space normal maps textures to a temporary mod folder
%Windir%\system32\robocopy "%cd%\Mod Extracted" "%cd%\Mod TSN" *_n.dds /s /nfl /ndl /njh /njs /ns /nc /np >>log.txt
rem copy the rest of the textures (ordinary textures) excluding normal map textures
%Windir%\system32\robocopy "%cd%\Mod Extracted" "%cd%\Mod DDS" *.dds /s  /xf *_n.dds *_msn.dds /nfl /ndl /njh /njs /ns /nc /np >>log.txt



goto finish


:exterior
rem if exterior textures are being separated from other textures for mods in "Mod Extracted" create new folders
IF NOT EXIST "%cd%\Mod Exterior Textures\NUL"  MD "Mod Exterior Textures"
IF NOT EXIST "%cd%\Mod Exterior Normal Maps\NUL"  MD "Mod Exterior Normal Maps"

IF NOT EXIST "%cd%\Mod Extracted" goto multires



rem copy all textures to "Mod TMP"
%Windir%\system32\robocopy "%cd%\Mod Extracted" "%cd%\Mod TMP" *.dds /s  /nfl /ndl /njh /njs /ns /nc /np >>log.txt

rem copy the non-exterior ordinary textures and terrain space normal maps
%Windir%\system32\robocopy "%cd%\Mod TMP" "%cd%\Mod DDS" *.dds /s  /xf *_n.dds *_msn.dds /xd *landscape *architecture /move /nfl /ndl /njh /njs /ns /nc /np >>log.txt
%Windir%\system32\robocopy "%cd%\Mod TMP" "%cd%\Mod TSN" *_n.dds  /s /xd *landscape *architecture /move /nfl /ndl /njh /njs /ns /nc /np >>log.txt
rem put model space normal maps in Mod Body Textures so uncompressed textures won't be corrupted or compressed
%Windir%\system32\robocopy "%cd%\Mod TMP" "%cd%\Mod Body Textures" *_msn.dds /s /move /nfl /ndl /njh /njs /ns /nc /np >>log.txt

rem now copy what remains, namely the exterior color map textures and tangent space normal maps
%Windir%\system32\robocopy "%cd%\Mod TMP" "%cd%\Mod Exterior Textures" *.dds /s  /xf *_n.dds /xd *characters /move /nfl /ndl /njh /njs /ns /nc /np >>log.txt
%Windir%\system32\robocopy "%cd%\Mod TMP" "%cd%\Mod Exterior Normal Maps" *_n.dds /s /move /nfl /ndl /njh /njs /ns /nc /np >>log.txt

rem "Mod TMP" should now be empty and can be deleted.
rd "%cd%\Mod TMP" /s /q /ndl /njh /njs /ns /nc /np >>log.txt

@echo on

:multires
IF NOT EXIST "%cd%\Higher Res Mod Files" goto finish


rem copy all non-texture files from the multi-resolution mods to the output folder
%Windir%\system32\robocopy "%cd%\Higher Res Mod Files" "%cd%\Mod Optimized" *.* /s  /xf *.dds /nfl /ndl /njh /njs /ns /nc /np >>log.txt

rem create temporary dds folder
IF NOT EXIST "%cd%\Lower Res DDS\NUL"  MD "Lower Res DDS"
IF NOT EXIST "%cd%\Low Res tmp\NUL" MD "Low Res tmp"
rem copy all lower res version textures to this temporary dds folder
%Windir%\system32\robocopy "%cd%\Lower Res Mod Files" "%cd%\Lower Res DDS" *.dds /s  /nfl /ndl /njh /njs /ns /nc /np >>log.txt
rem move all the non-exterior textures to a temporary folder
%Windir%\system32\robocopy "%cd%\Lower Res DDS" "%cd%\Low Res tmp" *.dds /s /xd *landscape *architecture /move /nfl /ndl /njh /njs /ns /nc /np >>log.txt
rem now that "Lower Res DDS" has only exterior textures, delete the temporary folder
IF EXIST "%cd%\Low Res tmp" rd "%cd%\Low Res tmp" /s /q >>log.txt


rem copy the non-exterior ordinary textures and tangent space normal maps from Higher Res Mod Files
%Windir%\system32\robocopy "%cd%\Higher Res Mod Files" "%cd%\Mod DDS" *.dds /s  /xf *_n.dds *_msn.dds /xd *landscape *architecture /nfl /ndl /njh /njs /ns /nc /np >>log.txt
%Windir%\system32\robocopy "%cd%\Higher Res Mod Files" "%cd%\Mod TSN" *_n.dds /s /xd *landscape *architecture /nfl /ndl /njh /njs /ns /nc /np >>log.txt
rem put model space normal maps in Mod Body Textures so uncompressed textures won't be corrupted or compressed
%Windir%\system32\robocopy "%cd%\Higher Res Mod Files" "%cd%\Mod Body Textures" *_msn.dds /s /nfl /ndl /njh /njs /ns /nc /np >>log.txt

rem now copy the exterior ordinary textures and tangent space normal maps from the Lower Res Mod Files
%Windir%\system32\robocopy "%cd%\Lower Res DDS" "%cd%\Mod Exterior Textures" *.dds /s  /xf *_n.dds /move /nfl /ndl /njh /njs /ns /nc /np >>log.txt
%Windir%\system32\robocopy "%cd%\Lower Res DDS" "%cd%\Mod Exterior Normal Maps" *_n.dds /s /move /nfl /ndl /njh /njs /ns /nc /np >>log.txt
rem "Lower Res DDS" should now be empty


:finish

rem The next commands handle body textures that typically have a mix of compressed and uncompressed textures.


rem move non-character non-interface-related textures in Mod DDS to Mod Ordinary Textures
%Windir%\system32\robocopy "%cd%\Mod DDS" "%cd%\Mod Ordinary Textures" *.dds /s  /xd *characters *interface *terminals *pipboy3000 *pimpboy3billion /move /nfl /ndl /njh /njs /ns /nc /np >>log.txt
rem copy the remaining textures in a "character" folder, to Mod Body Textures, leaving the interface textures
%Windir%\system32\robocopy "%cd%\Mod DDS" "%cd%\Mod Body Textures" *.dds /s /xd *interface *terminals *pipboy3000 *pimpboy3billion /move /nfl /ndl /njh /njs /ns /nc /np >>log.txt
rem now move the interface textures to "Mod Optimized" 
%Windir%\system32\robocopy "%cd%\Mod DDS" "%cd%\Mod Optimized" *.dds /s /move /nfl /ndl /njh /njs /ns /nc /np >>log.txt
rem Mod DDS is now empty

rem "Mod TSN" might still have some body textures. The normal maps that are not in a "characters" folder will be moved to "Mod Normal Maps"
%Windir%\system32\robocopy "%cd%\Mod TSN" "%cd%\Mod Normal Maps" *.dds /s  /xd *characters *interface *terminals *pipboy3000 *pimpboy3billion /move /nfl /ndl /njh /njs /ns /nc /np >>log.txt

rem now move the remaining body-related TSN textures to "Mod Body Textures" leaving the interface textures
%Windir%\system32\robocopy "%cd%\Mod TSN" "%cd%\Mod Body Textures" *.dds /s /xd *interface *terminals *pipboy3000 *pimpboy3billion /move /nfl /ndl /njh /njs /ns /nc /np >>log.txt

rem now move the interface textures to "Mod Optimized" 
%Windir%\system32\robocopy "%cd%\Mod TSN" "%cd%\Mod Optimized" *.dds /s /move /nfl /ndl /njh /njs /ns /nc /np >>log.txt
rem Mod DDS is now empty

rem now remove "Mod DDS" and "Mod TSN" temporary directories as they are empty and no longer needed
rd "%cd%\Mod DDS" /s /q /ndl /njh /njs /ns /nc /np >>log.txt
rd "%cd%\Mod TSN" /s /q /ndl /njh /njs /ns /nc /np >>log.txt



@echo off
REM When the batch file processing terminates, check the log file (log.txt). It should not show many errors.


rem get time of completion
set ENDTIME=%TIME%

rem from http://stackoverflow.com/questions/9922498/calculate-time-difference-in-batch-file

rem Change formatting for the start and end times
for /F "tokens=1-4 delims=:.," %%a in ("%STARTTIME%") do (
    set /A "start=(((%%a*60)+1%%b %% 100)*60+1%%c %% 100)*100+1%%d %% 100"
)

for /F "tokens=1-4 delims=:.," %%a in ("%ENDTIME%") do (
    set /A "end=(((%%a*60)+1%%b %% 100)*60+1%%c %% 100)*100+1%%d %% 100"
)

rem Calculate the elapsed time by subtracting values
set /A elapsed=end-start

rem Format the results for output
set /A hh=elapsed/(60*60*100), rest=elapsed%%(60*60*100), mm=rest/(60*100), rest%%=60*100, ss=rest/100, cc=rest%%100
if %hh% lss 10 set hh=0%hh%
if %mm% lss 10 set mm=0%mm%
if %ss% lss 10 set ss=0%ss%
if %cc% lss 10 set cc=0%cc%

set DURATION=%hh%:%mm%:%ss%,%cc%


echo.
echo.
echo Start    : %STARTTIME%
echo Finish   : %ENDTIME%
echo          ---------------
echo Duration : %DURATION%

echo.
echo The batch file processing is complete. Check the log file (log.txt) for errors with
echo ERROR level greater than 3.
echo Ignore any ERROR Level 2 errors (The system cannot find the file specified).
echo These are not actually errors.
echo.
pause


